@@ -18,7 +18,7 @@ use File::Path ();
 use Cwd ();
 use Config;
 
-our $VERSION = '0.900';
+our $VERSION = '0.901';
 
 use constant WIN32 => $^O eq 'MSWin32';
 
@@ -214,7 +214,6 @@ sub cmd_install {
         (exists $self->{target_perl} ? (target_perl => $self->{target_perl}) : ()),
     );
 
-    # dryrun
     my ($packages, $dists) = $self->initial_job($master);
     return 0 unless $packages;
 
@@ -234,15 +233,29 @@ sub cmd_install {
         ($self->{global} ? () : (local_lib => $self->{local_lib})),
     );
 
-    {
-        last unless $] < 5.016;
-        my $requirements = [
-            { package => 'ExtUtils::MakeMaker', version_range => '6.58' },
-            { package => 'ExtUtils::ParseXS',   version_range => '3.16' },
-        ];
-        my ($is_satisfied, @need_resolve) = $master->is_satisfied($requirements);
-        last if $is_satisfied;
-        $master->add_job(type => "resolve", %$_) for @need_resolve;
+    if ($] < 5.016) {
+        my %toolchain = ('ExtUtils::MakeMaker' => '6.58', 'ExtUtils::ParseXS' => '3.16');
+        for my $name (sort keys %toolchain) {
+            my ($req, $i);
+            my $req_range = $toolchain{$name};
+            if ( ($i) = grep { $packages->[$_]{package} eq $name } 0..$#{$packages} ) {
+                my $user = $packages->[$i];
+                my $range = eval { App::cpm::version::range_merge($user->{version_range}, $req_range) };
+                die sprintf "We have to install %s %s or later first, but you requested that of %s\n",
+                    $name, $req_range, $user->{version_range} if $@;
+                $req = { package => $name, version_range => $range, dev => $user->{dev} };
+            } else {
+                $req = { package => $name, version_range => $req_range };
+            }
+            my ($is_satisfied, @need_resolve) = $master->is_satisfied([$req]);
+            if ($is_satisfied) {
+                # nothing to do
+            } else {
+                $master->add_job(type => "resolve", %$_) for @need_resolve;
+                splice @$packages, $i, 1 if defined $i;
+            }
+        }
+
         $self->install($master, $worker, 1);
         if (my $fail = $master->fail) {
             local $App::cpm::Logger::VERBOSE = 0;
