@@ -2,7 +2,7 @@ package App::cpm::Worker;
 use strict;
 use warnings;
 use utf8;
-our $VERSION = '0.900';
+our $VERSION = '0.901';
 
 use App::cpm::Worker::Installer;
 use App::cpm::Worker::Resolver;
@@ -28,7 +28,11 @@ sub new {
 
 sub prebuilt_base {
     my ($class, $home) = @_;
-    my $digest = substr Digest::MD5::md5_hex(Config->myconfig), 0, 8;
+
+    # XXX Taking account of relocatable perls, we also use $Config{startperl}
+    my $identity = $Config{startperl} . Config->myconfig;
+    my $digest = Digest::MD5::md5_hex($identity);
+    $digest = substr $digest, 0, 8;
     "$home/builds/$Config{version}-$Config{archname}-$digest";
 }
 
